import { BatchResult } from 'src/core/response/domain/ok-response.interface';
import { BaseManager } from '../base.manager';
import { HttpStatus, NotFoundException } from '@nestjs/common';
import { ValidateRelationHelper } from 'src/core/helpers/validation/validate-relation.helper';

export abstract class BaseBatchDeleteManager<Entity> extends BaseManager {
  protected dataIds: string[];
  protected result: BatchResult;
  abstract get entityTarget(): any;

  setData(ids: string[]): void {
    this.dataIds = ids;
  }

  validateProcess(): Promise<void> {
    return;
  }

  prepareData(): Promise<void> {
    return;
  }

  async process(): Promise<void> {
    let totalFailed = 0;
    let totalSuccess = 0;
    let messages = [];

    for (const id of this.dataIds) {
      try {
        const entity = await this.dataService.getOneByOptions({
          where: {
            id: id,
          },
        });

        if (!entity) {
          throw new NotFoundException({
            statusCode: HttpStatus.NOT_FOUND,
            message: `Failed! Entity with id ${id} not found`,
            error: 'Entity Not Found',
          });
        }

        await this.validateData(entity);
        await new ValidateRelationHelper(
          id,
          this.dataService,
          this.validateRelations,
          this.tableName,
        ).execute();

        await this.dataService.deleteById(
          this.queryRunner,
          this.entityTarget,
          id,
        );

        totalSuccess = totalSuccess + 1;
      } catch (error) {
        totalFailed = totalFailed + 1;
        messages.push(error.response?.message ?? error.message);
      }
    }

    this.result = {
      total_items: this.dataIds.length,
      total_failed: totalFailed,
      total_success: totalSuccess,
      messages: messages,
    };
  }

  abstract validateData(data: Entity): Promise<void>;
  abstract getResult(): BatchResult;
}
