import { HttpStatus, UnprocessableEntityException } from '@nestjs/common';
import { BaseManager } from '../base.manager';
import { ValidateRelationHelper } from 'src/core/helpers/validation/validate-relation.helper';

export abstract class BaseDeleteManager<Entity> extends BaseManager {
  protected dataId: string;
  protected result: Entity;
  abstract get entityTarget(): any;

  setData(id: string): void {
    this.dataId = id;
  }

  async prepareData(): Promise<void> {
    this.data = await this.dataService.getOneByOptions({
      where: {
        id: this.dataId,
      },
    });

    if (!this.data)
      throw new UnprocessableEntityException({
        statusCode: HttpStatus.UNPROCESSABLE_ENTITY,
        message: `Data with id ${this.dataId} not found`,
        error: 'Unprocessable Entity',
      });

    return;
  }

  async process(): Promise<void> {
    await new ValidateRelationHelper(
      this.dataId,
      this.dataService,
      this.validateRelations,
      this.tableName,
    ).execute();

    await this.dataService.deleteById(
      this.queryRunner,
      this.entityTarget,
      this.dataId,
    );
  }

  abstract getResult(): string;
}
