import { ValidateRelationHelper } from 'src/core/helpers/validation/validate-relation.helper';
import { BaseManager } from '../base.manager';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { UserPrivilegeModel } from 'src/modules/user-related/user-privilege/data/models/user-privilege.model';

export abstract class BaseUpdateStatusManager<Entity> extends BaseManager {
  protected dataId: string;
  protected result: Entity;
  protected dataStatus: STATUS;
  protected duplicateColumn: string[];
  abstract get entityTarget(): any;

  setData(id: string, status: STATUS): void {
    this.dataId = id;
    this.dataStatus = status;
  }

  async prepareData(): Promise<void> {
    this.data = new UserPrivilegeModel();

    Object.assign(this.data, {
      editor_id: this.user.id,
      editor_name: this.user.name,
      updated_at: new Date().getTime(),
      id: this.dataId,
      status: this.dataStatus,
    });
  }

  async process(): Promise<void> {
    await new ValidateRelationHelper(
      this.dataId,
      this.dataService,
      this.validateRelations,
      this.tableName,
    ).execute();

    this.result = await this.dataService.update(
      this.queryRunner,
      this.entityTarget,
      { id: this.dataId },
      this.data,
    );
  }

  abstract getResult(): string;
}
