import { CheckDuplicateHelper } from 'src/core/helpers/query/check-duplicate.helpers';
import { BaseManager } from '../base.manager';
import { ValidateRelationHelper } from 'src/core/helpers/validation/validate-relation.helper';
import { columnUniques } from 'src/core/strings/constants/interface.constants';

export abstract class BaseUpdateManager<Entity> extends BaseManager {
  protected dataId: string;
  protected result: Entity;
  protected duplicateColumn: string[];
  abstract get entityTarget(): any;
  abstract get uniqueColumns(): columnUniques[];

  setData(id: string, entity: Entity): void {
    this.dataId = id;
    this.data = entity;
  }

  async prepareData(): Promise<void> {
    Object.assign(this.data, {
      editor_id: this.user.id,
      editor_name: this.user.name,
      updated_at: new Date().getTime(),
    });

    if (this.uniqueColumns.length) {
      await new CheckDuplicateHelper(
        this.dataService,
        this.tableName,
        this.uniqueColumns,
        this.data,
        this.dataId,
      ).execute();
    }
  }

  async process(): Promise<void> {
    await new ValidateRelationHelper(
      this.dataId,
      this.dataService,
      this.validateRelations,
      this.tableName,
    ).execute();

    this.result = await this.dataService.update(
      this.queryRunner,
      this.entityTarget,
      { id: this.dataId },
      this.data,
    );
  }

  async getResult(): Promise<Entity> {
    return await this.dataService.getOneByOptions({
      where: {
        id: this.dataId,
      },
    });
  }
}
